@@ -16,9 +16,14 @@
  * -----------------------------------------------------------------
  */
 
+/* RGF: printf is not allowed */
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <R.h>
+#undef printf
+#define printf Rprintf
+
 #include <nvector/nvector_serial.h>
 #include <sundials/sundials_math.h>
 
