@@ -1,13 +1,6 @@
 /*
  * The GeoSSE equations, implemented in C
- *
  * By Emma Goldberg <eeg@uic.edu>
- *
- * Usage outside of the package:
- *    $ gcc -c -I/usr/share/R/include -fPIC geosse-eqs.c
- *    $ gcc -shared -o geosse-eqs.so geosse-eqs.o
- *    creates geosse-eqs.so
- *    > dyn.load('geosse-eqs.so')   # load in R
  */
 
 #include <R.h>
@@ -78,16 +71,15 @@ void initmod_geosse(void (* odeparms)(int *, double *))
 }
 
 void derivs_geosse(int *neq, double *t, double *y, double *ydot, 
-		  double *yout, int *ip) {
+		   double *yout, int *ip) {
   do_derivs_geosse(parms_geosse, y, ydot);
 }
 
 /* CVODES */
 int derivs_geosse_cvode(realtype t, N_Vector y, N_Vector ydot,
-		       void *user_data) {
+			void *user_data) {
   do_derivs_geosse(((UserData*) user_data)->p,
-		   NV_DATA_S(y),
-		   NV_DATA_S(ydot));
+		   NV_DATA_S(y), NV_DATA_S(ydot));
   return 0;
 }
 
@@ -109,3 +101,51 @@ void initial_conditions_geosse(int neq, double *vars_l, double *vars_r,
   vars_out[4] = vars_l[4] * vars_r[4] * pars[0];
   vars_out[5] = vars_l[5] * vars_r[5] * pars[1];
 }
+
+
+/*** For split ***/
+
+/* Auxilliary (just compute E) */
+
+void initmod_geosse_aux(void (* odeparms)(int *, double *))
+{
+  int N = 7;
+  odeparms(&N, parms_geosse);
+}
+
+void do_derivs_geosse_aux(double *pars, double *y, double *ydot)
+{
+  double E_1 = y[0];
+  double E_2 = y[1];
+  double E_3 = y[2];
+
+  double sA = pars[0], sB = pars[1], sAB = pars[2], 
+         xA = pars[3], xB = pars[4], dA = pars[5], dB = pars[6]; 
+
+  /*  dE_1 / dt  */
+  ydot[0] = -(sA + sB + xA + xB + sAB) * E_1 
+            + xA * E_3 + xB * E_2 
+      + sA * E_1 * E_2 + sB * E_1 * E_3 + sAB * E_2 * E_3;
+
+  /*  dE_2 / dt  */
+  ydot[1] = -(sA + dA + xA) * E_2 
+            + xA + dA * E_1 + sA * E_2 * E_2;
+
+  /*  dE_3 / dt  */
+  ydot[2] = -(sB + dB + xB) * E_3 
+            + xB + dB * E_1 + sB * E_3 * E_3;
+}
+
+/* deSolve */
+void derivs_geosse_aux(int *neq, double *t, double *y, double *ydot, 
+                       double *yout, int *ip) {
+  do_derivs_geosse_aux(parms_geosse, y, ydot);
+}
+
+/* CVODES */
+int derivs_geosse_aux_cvode(realtype t, N_Vector y, N_Vector ydot,
+                           void *user_data) {
+  do_derivs_geosse_aux(((UserData*) user_data)->p,
+                      NV_DATA_S(y), NV_DATA_S(ydot));
+  return 0;
+}
